package handlers

import (
	"errors"
	"fmt"
	"github.com/gofiber/fiber/v2"
	"github.com/jackc/pgx/v5"
	"lmika.dev/lmika/hugo-cms/models"
	"lmika.dev/lmika/hugo-cms/services/sites"
	"net/http"
	"time"
)

type Site struct {
	Site *sites.Service
}

func (s *Site) Create(c *fiber.Ctx) error {
	user := GetUser(c)

	site, err := s.Site.CreateSite(c.UserContext(), user, "New Site "+time.Now().Format("2006-01-02 15:04:05"))
	if err != nil {
		return err
	}

	UpdatePrefCookie(c, func(prefs *models.PrefCookie) {
		prefs.SiteID = site.ID
	})

	return c.Redirect(fmt.Sprintf("/sites/%v/posts", site.ID))
}

func (s *Site) Show(c *fiber.Ctx) error {
	id, err := c.ParamsInt("siteId")
	if err != nil {
		return err
	}

	site, err := s.Site.GetSite(c.UserContext(), id)
	if err != nil {
		return err
	}

	return c.Render("sites/index", fiber.Map{
		"site": site,
	}, "layouts/main")
}

func (s *Site) Settings(c *fiber.Ctx) error {
	return c.Render("sites/settings", fiber.Map{
		"themes": s.Site.Themes(),
	}, "layouts/site")
}

func (s *Site) SaveSettings(c *fiber.Ctx) error {
	site := GetSite(c)

	var req sites.NewSettings
	if err := c.BodyParser(&req); err != nil {
		return err
	}

	if err := s.Site.SaveSettings(c.Context(), site, req); err != nil {
		return err
	}

	return c.Redirect(fmt.Sprintf("/sites/%v/settings", site.ID))
}

func (s *Site) Rebuild(c *fiber.Ctx) error {
	if err := s.Site.Rebuild(c.UserContext(), GetSite(c)); err != nil {
		return err
	}

	return c.Redirect(fmt.Sprintf("/sites/%v/posts", GetSite(c).ID))
}

func (s *Site) WithSite() fiber.Handler {
	return func(c *fiber.Ctx) (err error) {
		id, err := c.ParamsInt("siteId")
		if err != nil {
			return err
		}

		site, err := s.Site.GetSite(c.UserContext(), id)
		if err != nil {
			return err
		}

		user := GetUser(c)
		if site.OwnerUserID != user.ID {
			return c.Status(http.StatusForbidden).SendString("not permitted")
		}

		c.Locals("site", site)

		if prodTarget, err := s.Site.GetProdTargetOfSite(c.UserContext(), int(site.ID)); err == nil {
			c.Locals("prodTarget", prodTarget)
		} else if !errors.Is(err, pgx.ErrNoRows) {
			return err
		}

		return c.Next()
	}
}
